#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Download, parse and prepare the latest annual Consumer Price Index (CPI) dataset.

Source: The Bureau of Labor Statistics at the U.S. Department of Labor.
Survey: CPI-All Urban Consumers (Current Series)
        All items in U.S. city average, all urban consumers, not seasonally adjusted
Identifer: CUUR0000SA0
"""
import os
import csv
import requests


def get_tsv():
    """
    Download the latest annual Consumer Price Index (CPI) dataset.
    """
    bls_url = "https://download.bls.gov/pub/time.series/cu/cu.data.1.AllItems"
    response = requests.get(bls_url)
    this_dir = os.path.dirname(__file__)
    with open(os.path.join(this_dir, 'data.tsv'), 'w') as f:
        f.write(response.text)


def parse_tsv():
    """
    Parse the downloaded fixed-width file from the BLS and convert it into a CSV.
    """
    this_dir = os.path.dirname(__file__)

    # Get the raw data
    input = os.path.join(this_dir, "data.tsv")
    reader = csv.DictReader(open(input, 'r'), delimiter="\t")

    # Figure out where we're going to store the clean data
    output = os.path.join(this_dir, "data.csv")
    writer = csv.DictWriter(open(output, 'w'), fieldnames=["series", "period", "year", "value"])
    writer.writeheader()

    # Loop through it
    for row in reader:
        # Clean it up
        d = dict(
            series=row['series_id'].strip(),
            period=row['period'].strip(),
            year=int(row['year'].strip()),
            value=float(row['value'].strip())
        )
        # Only keep the annual totals (M13) from the series we care about.
        if d['series'] == 'CUUR0000SA0' and d['period'] == 'M13':
            writer.writerow(d)


if __name__ == '__main__':
    get_tsv()
    parse_tsv()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Open the Consumer Price Index (CPI) dataset.
"""
import os
import csv


def _get_cpi_dict():
    """
    Returns a dictionary of the CPI-U adjustment value for each year available.
    """
    # Open up the CSV from the BLS
    this_dir = os.path.dirname(__file__)
    csv_path = os.path.join(this_dir, 'data.csv')
    csv_file = open(csv_path, "r")
    csv_data = csv.DictReader(csv_file)

    # Convert it into a dictionary and pass it out.
    return dict(
        (int(r['year']), float(r['value'])) for r in csv_data
    )


cpi_by_year = _get_cpi_dict()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Quickly adjust U.S. dollars for inflation using the Consumer Price Index (CPI)
"""
from .data import cpi_by_year
from .errors import CPIDoesNotExist


def get(year):
    """
    Returns the CPI value for a given year.
    """
    try:
        return cpi_by_year[year]
    except KeyError:
        raise CPIDoesNotExist("CPI value not found for {}".format(year))


def inflate(value, year, to=None):
    """
    Returns a dollar value adjusted for inflation.

    You must submit the value, followed by the year its from.

    By default, the input is adjusted to the most recent year available from the CPI.

    If you'd like to adjust to a different year, submit it as an integer to the optional `to` keyword argument.
    """
    # Figure out the year we're inflating to.
    if not to:
        # If an input hasn't been provided, we'll use the maximum year
        max_year = max(cpi_by_year.keys())
        to = max_year

    # If the two years match, just return the value unadjusted
    if year == to:
        return value

    # Otherwise, let's do the math.
    # The input value is multiplied by the CPI of the target year,
    # Then divided into the cpi from the source year.
    return (value * get(to)) / float(get(year))
